import Schema from 'async-validator';
import _clone from 'lodash/clone';
import _setWith from 'lodash/setWith';
import _get from 'lodash/get';
import _find from 'lodash/find';
import { __assign, __rest } from 'tslib';
import React__default, { useReducer, useCallback, useContext, useEffect, useMemo, createContext } from 'react';
import classNames from 'classnames';

function init(initialValue) {
    return { value: initialValue, error: {} };
}
function reducer(state, action) {
    var value = state.value, error = state.error;
    var type = action.type, payload = action.payload;
    switch (type) {
        case 'setValue':
            value = _clone(value);
            Object.keys(payload).forEach(function (key) {
                _setWith(value, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'setError':
            error = _clone(error);
            Object.keys(payload).forEach(function (key) {
                _setWith(error, key, payload[key], _clone);
            });
            return { value: value, error: error };
        case 'reset':
            return init(payload);
        default:
            throw new Error();
    }
}
function useFormx(initialValue, rules) {
    if (initialValue === void 0) { initialValue = {}; }
    var _a = useReducer(reducer, initialValue, init), formState = _a[0], dispatch = _a[1];
    var setFieldsValue = useCallback(function (payload) {
        dispatch({ type: 'setValue', payload: payload });
    }, []);
    var setFieldsError = useCallback(function (payload) {
        dispatch({ type: 'setError', payload: payload });
    }, []);
    var resetFields = useCallback(function () {
        dispatch({ type: 'reset', payload: initialValue });
    }, []);
    var getRules = function (prop, trigger) {
        if (!rules || !rules[prop]) {
            return undefined;
        }
        var result = Array.isArray(rules[prop]) ? rules[prop] : [rules[prop]];
        if (!trigger) {
            return result;
        }
        return result.filter(function (rule) {
            return !rule.trigger || rule.trigger.indexOf(trigger) !== -1;
        });
    };
    var validate = function (keys, trigger) {
        var allKeys = Object.keys(formState.value);
        var validKeys = keys ? keys.filter(function (v) { return allKeys.indexOf(v) !== -1; }) : allKeys;
        var descriptor = {};
        var data = {};
        validKeys.forEach(function (key) {
            var rule = getRules(key, trigger);
            var value = _get(formState.value, key);
            data[key] = value;
            if (rule && rule.length) {
                descriptor[key] = rule;
            }
        });
        if (Object.keys(descriptor).length === 0) {
            return Promise.resolve(data);
        }
        return new Promise(function (resolve, reject) {
            var validator = new Schema(descriptor);
            validator.validate(data, function (errors) {
                var error = {};
                validKeys.forEach(function (key) {
                    error[key] = errors && _find(errors, function (o) { return o.field === key; });
                });
                setFieldsError(error);
                if (errors) {
                    reject(error);
                }
                else {
                    resolve(data);
                }
            });
        });
    };
    var bindFormx = function (prop) {
        return {
            setFieldsValue: setFieldsValue,
            validate: validate,
            getRules: getRules,
            prop: prop,
            key: prop,
            value: _get(formState.value, prop),
            error: _get(formState.error, prop)
        };
    };
    return __assign({}, formState, { bindFormx: bindFormx, validate: validate, setFieldsValue: setFieldsValue, setFieldsError: setFieldsError, resetFields: resetFields });
}

var FormxContext = createContext({});

function isObject(obj) {
    return obj !== null && typeof obj === 'object';
}
function getValue(event) {
    if (!isObject(event) || !event.target) {
        return event;
    }
    var target = event.target;
    var value = target.type === 'checkbox' ? target.checked : target.value;
    return value;
}
function getStyle(labelPosition, labelWidth) {
    var label = {};
    var content = {};
    if (labelPosition === 'left' || labelPosition === 'right') {
        label.textAlign = labelPosition;
        if (labelWidth) {
            label.width = labelWidth;
            content.marginLeft = labelWidth;
        }
    }
    return { label: label, content: content };
}
function FormxItem(props) {
    var prop = props.prop, value = props.value, error = props.error, getRules = props.getRules, setFieldsValue = props.setFieldsValue, validate = props.validate, label = props.label, labelStyle = props.labelStyle, _a = props.trigger, trigger = _a === void 0 ? 'onChange' : _a, _b = props.valuePropName, valuePropName = _b === void 0 ? 'value' : _b, children = props.children, rest = __rest(props, ["prop", "value", "error", "getRules", "setFieldsValue", "validate", "label", "labelStyle", "trigger", "valuePropName", "children"]);
    var _c = useContext(FormxContext), labelPosition = _c.labelPosition, labelSuffix = _c.labelSuffix, labelWidth = _c.labelWidth, disabled = _c.disabled;
    useEffect(function () {
        return function () {
            var _a;
            if (setFieldsValue && prop) {
                setFieldsValue((_a = {}, _a[prop] = undefined, _a));
            }
        };
    }, []);
    var message = error && error.message;
    var handleChange = function (event) {
        var _a;
        var value = getValue(event);
        if (setFieldsValue && prop) {
            setFieldsValue((_a = {}, _a[prop] = value, _a));
            if (validate) {
                validate([prop], 'change');
            }
        }
    };
    var handleBlur = function () {
        if (setFieldsValue && prop && validate) {
            validate([prop], 'blur');
        }
    };
    var isRequired = function () {
        var data = prop && getRules && getRules(prop);
        if (data) {
            data = [].concat(data);
            for (var index = 0; index < data.length; index++) {
                var rule = data[index];
                if (rule.required) {
                    return true;
                }
            }
        }
        return false;
    };
    var itemClasses = classNames('easy-formx-item', {
        'has-error': error,
        'easy-formx-item--with-help': message
    });
    var labelClasses = classNames('easy-formx-item__label', {
        'is-required': isRequired()
    });
    var style = getStyle(labelPosition, labelWidth);
    var items = children;
    if (prop) {
        items = React__default.Children.map(children, function (child, index) {
            var _a;
            if (index === 0 && React__default.isValidElement(child)) {
                return React__default.cloneElement(child, (_a = {
                        disabled: child.props.disabled || disabled
                    },
                    _a[valuePropName] = value,
                    _a[trigger] = handleChange,
                    _a));
            }
            return child;
        });
    }
    return (React__default.createElement("div", __assign({}, rest, { className: itemClasses, onBlur: handleBlur }),
        label && (React__default.createElement("label", { className: labelClasses, style: __assign({}, style.label, labelStyle) },
            label,
            React__default.createElement("span", { className: "esay-formx-item__suffix" }, labelSuffix))),
        React__default.createElement("div", { className: "easy-formx-item__content", style: style.content },
            items,
            message && React__default.createElement("div", { className: "easy-formx-item__error" }, message))));
}

var Provider = FormxContext.Provider;
function Formx(props) {
    var _a = props.labelPosition, labelPosition = _a === void 0 ? 'right' : _a, _b = props.labelSuffix, labelSuffix = _b === void 0 ? ':' : _b, labelWidth = props.labelWidth, disabled = props.disabled, className = props.className, restProps = __rest(props, ["labelPosition", "labelSuffix", "labelWidth", "disabled", "className"]);
    var formProps = { labelPosition: labelPosition, labelWidth: labelWidth, labelSuffix: labelSuffix, disabled: disabled };
    var memoized = useMemo(function () {
        return formProps;
    }, [JSON.stringify(formProps)]);
    var formClasses = classNames('easy-formx', "easy-formx--label-" + labelPosition, className);
    return (React__default.createElement(Provider, { value: memoized },
        React__default.createElement("form", __assign({ className: formClasses }, restProps))));
}

export { useFormx, FormxItem, Formx };
//# sourceMappingURL=index.es.js.map
